use crate::{State, assets::initial_context, get_lang, get_user_from_token};
use axum::{
    extract::Path,
    response::{Html, IntoResponse},
    Extension,
};
use axum_extra::extract::CookieJar;
use tetratto_core::model::{Error, auth::ConnectionService};

use super::render_error;

/// `/auth/login`
pub async fn login_request(jar: CookieJar, Extension(data): Extension<State>) -> impl IntoResponse {
    let data = data.read().await;
    let user = get_user_from_token!(jar, data.0);

    // if user.is_some() {
    //     return Err(Redirect::to("/"));
    // }

    let lang = get_lang!(jar, data.0);
    let context = initial_context(&data.0.0, lang, &user).await;

    Html(data.1.render("auth/login.html", &context).unwrap())
}

/// `/auth/register`
pub async fn register_request(
    jar: CookieJar,
    Extension(data): Extension<State>,
) -> impl IntoResponse {
    let data = data.read().await;
    let user = get_user_from_token!(jar, data.0);

    // if user.is_some() {
    //     return Err(Redirect::to("/"));
    // }

    let lang = get_lang!(jar, data.0);
    let context = initial_context(&data.0.0, lang, &user).await;

    Html(data.1.render("auth/register.html", &context).unwrap())
}

/// `/auth/connections_link/{service}`
pub async fn connection_callback_request(
    jar: CookieJar,
    Extension(data): Extension<State>,
    Path(service): Path<ConnectionService>,
) -> impl IntoResponse {
    let data = data.read().await;
    let user = match get_user_from_token!(jar, data.0) {
        Some(ua) => ua,
        None => return Html(render_error(Error::NotAllowed, &jar, &data, &None).await),
    };

    let lang = get_lang!(jar, data.0);
    let mut context = initial_context(&data.0.0, lang, &Some(user)).await;

    context.insert("connection_type", &service);
    Html(data.1.render("auth/connection.html", &context).unwrap())
}
